@@ -2,43 +2,41 @@ package HTTP::UA::Parser;
 use strict;
 use warnings;
 use YAML::Tiny 'LoadFile';
-our $VERSION = '0.005';
-my ($REGEX,$PATH);
+our $VERSION = '0.006';
+my ($REGEX, $PATH, $PARSER);
 my $PACKAGE = __PACKAGE__;
 
 sub new {
-    my ($class,$op) = @_;
-    my ($ua,$path);
+    my ($class, $op) = @_;
+    my ($ua, $path);
     if (ref $op eq 'HASH'){
-	$path = $op->{regex};
-	$ua = $op->{ua};
+        $path = $op->{regex};
+        $ua = $op->{ua};
     } else { $ua = $op; }
     
     if (!$REGEX){
-	if ($path){
-	    $REGEX ||= LoadFile( $path );
-	} else {
-	    $PATH = HTTP::UA::Parser::Utils::getPath();
-	    my $regFile;
-	    if (-e ($regFile = $PATH.'../../../../../../regexes.yaml')){}
-	    elsif (-e ($regFile = $PATH.'/regexes.yaml')){}
-	    else {
-		die
-		"Can't find regexes.yaml file\n".
-		"you can download/update it using command line by typing\n".
-		"    % ua_parser -u\n".
-		"or simply download it from\n".
-		"https://raw.github.com/tobie/ua-parser/master/regexes.yaml".
-		"and include it as an option when construct new HTTP::UA::Parser class\n".
-		"ex ->new({regex => '/full/path/to/regexes.yaml'})";
-	    }
-	    $REGEX = LoadFile( $regFile );
-	}
+        if ($path){
+            $REGEX ||= LoadFile( $path );
+        } else {
+            $PATH = HTTP::UA::Parser::Utils::getPath();
+            my $regFile;
+            unless (-e ($regFile = $PATH.'/regexes.yaml')){
+                die
+                "Can't find regexes.yaml file\n".
+                "you can download/update it using command line by typing\n".
+                "    % ua_parser -u\n".
+                "or simply download it from\n".
+                "https://raw.githubusercontent.com/ua-parser/uap-core/master/regexes.yaml".
+                "and include it as an option when construct new HTTP::UA::Parser class\n".
+                "ex ->new({regex => '/full/path/to/regexes.yaml'})";
+            }
+            $REGEX = LoadFile( $regFile );
+        }
     }
     
     my $self = {
         user_agent => $ua || $ENV{HTTP_USER_AGENT},
-	path => $PATH
+        path => $PATH
     };
     
     return bless($self,$class);
@@ -81,15 +79,14 @@ sub new {HTTP::UA::Parser::Base::new(@_)}
 sub parse {
     my $self = shift;
     my $ua = shift;
-    my $regexes = $REGEX->{user_agent_parsers};
-    my $parser = $self->makeParser($regexes);
+    my $parser = $PARSER->{ua} || ($PARSER->{ua} = $self->makeParser($REGEX->{user_agent_parsers}));
     return $parser->($ua);
 }
 
 sub makeParser {
     my $self = shift;
     my $regexes = shift;
-    return HTTP::UA::Parser::Utils::makeParser($regexes,\&_makeParsers);
+    return HTTP::UA::Parser::Utils::makeParser($regexes, \&_makeParsers);
 }
 
 sub _makeParsers {
@@ -100,10 +97,11 @@ sub _makeParsers {
     my $majorRep = $obj->{v1_replacement};
     my $minorRep = $obj->{v2_replacement};
     my $patchRep = $obj->{v3_replacement};
+    my $qr = HTTP::UA::Parser::Utils::regex($regexp);
     
     my $parser = sub {
         my $str = shift;
-        my @m = HTTP::UA::Parser::Utils::exe( $regexp , $str );
+        my @m = $str =~ $qr;
         if (!@m) { return undef; }
         my $family = defined $famRep ? HTTP::UA::Parser::Utils::replace($famRep,qr/\$1/,$m[0]) : $m[0];
         my $major = defined $majorRep ?  $majorRep : $m[1];
@@ -125,15 +123,14 @@ sub new {HTTP::UA::Parser::Base::new(@_)}
 sub parse {
     my $self = shift;
     my $ua = shift;
-    my $regexes = $REGEX->{os_parsers};
-    my $parser = $self->makeParser($regexes);
+    my $parser = $PARSER->{os} || ($PARSER->{os} = $self->makeParser($REGEX->{os_parsers}));
     return $parser->($ua);
 }
 
 sub makeParser {
     my $self = shift;
     my $regexes = shift;
-    return HTTP::UA::Parser::Utils::makeParser($regexes,\&_makeParsers);
+    return HTTP::UA::Parser::Utils::makeParser($regexes, \&_makeParsers);
 }
 
 sub _makeParsers {
@@ -145,10 +142,11 @@ sub _makeParsers {
     my $minorRep = $obj->{os_v2_replacement};
     my $patchRep = $obj->{os_v3_replacement};
     my $patchMinorRep = $obj->{os_v4_replacement};
+    my $qr = HTTP::UA::Parser::Utils::regex($regexp);
     
     my $parser = sub {
         my $str = shift;
-        my @m = HTTP::UA::Parser::Utils::exe( $regexp , $str );
+        my @m = $str =~ $qr;
         if (!@m) { return undef; }
         my $family = $famRep ? HTTP::UA::Parser::Utils::replace($famRep,qr/\$1/,$m[0]) : $m[0];
         my $major = defined $majorRep ? $majorRep : $m[1];
@@ -166,35 +164,77 @@ sub _makeParsers {
 ##=============================================================================
 package HTTP::UA::Parser::Device;
 
-sub new {HTTP::UA::Parser::Base::new(@_)}
+sub new {
+    my $class = shift;
+    my $self = {
+        family => $_[0] || 'Other',
+        brand => $_[1],
+        model => $_[2]
+    };
+    return bless($self, 'HTTP::UA::Parser::Base');
+}
 
-sub makeParser {
+sub toString {
     my $self = shift;
-    my $regexes = shift;
-    return HTTP::UA::Parser::Utils::makeParser($regexes,\&_makeParsers);
+    return $self->family;
 }
 
 sub parse {
     my $self = shift;
     my $ua = shift;
-    my $regexes = $REGEX->{device_parsers};
-    my $parser = $self->makeParser($regexes);
+    my $parser = $PARSER->{device} || ($PARSER->{device} = $self->makeParser($REGEX->{device_parsers}));
     return $parser->($ua);
 }
 
+sub makeParser {
+    my $self = shift;
+    my $regexes = shift;
+    my $makeParser = shift || \&_makeParsers;
+    my @parsers = map {
+        $makeParser->($_);
+    } @{$regexes};
+  
+    my $parser = sub {
+        my $ua = shift;
+        my @obj;
+        foreach my $parser (@parsers){
+            @obj = $parser->($ua);
+            return HTTP::UA::Parser::Device->new(@obj) if $obj[0];
+        }    
+    
+        HTTP::UA::Parser::Device->new();
+    };
+  
+    return $parser;
+}
+
 sub _makeParsers {
     my ($obj) = shift;
     my $regexp = $obj->{regex};
+    my $regexp_flag = $obj->{regex_flag};
     my $deviceRep = $obj->{device_replacement};
+    my $brandRep = $obj->{brand_replacement};
+    my $modelRep = $obj->{model_replacement};
+    my $qr = HTTP::UA::Parser::Utils::regex($regexp, $regexp_flag);
+    
     my $parser = sub {
         my $str = shift;
-        my @m = HTTP::UA::Parser::Utils::exe( $regexp , $str );
+        my @m = $str =~ $qr;
         if (!@m) { return undef; }
-        my $family = $deviceRep ? HTTP::UA::Parser::Utils::replace($deviceRep,qr/\$1/,$m[0]) : $m[0];
-        return ($family);
+        my $family = $deviceRep ? HTTP::UA::Parser::Utils::multiReplace($deviceRep, \@m) 
+                                : ($m[0] eq "1" ? undef : $m[0]);
+
+        my $brand  = $brandRep  ? HTTP::UA::Parser::Utils::multiReplace($brandRep, \@m)  
+                                : undef;
+
+        my $model  = $modelRep  ? HTTP::UA::Parser::Utils::multiReplace($modelRep, \@m)  
+                                : ($m[0] eq "1" ? undef : $m[0]);
+        
+        return ($family, $brand, $model);
     };
     return $parser;
 }
+
 ##=============================================================================
 ## Stringify Package
 ##=============================================================================
@@ -203,11 +243,11 @@ package HTTP::UA::Parser::Base;
 sub new {
     my $class = shift;
     my $self = {
-	family => $_[0] || 'Other',
-	major => $_[1],
-	minor => $_[2],
-	patch => $_[3],
-	patchMinor => $_[4]
+        family => $_[0] || 'Other',
+        major => $_[1],
+        minor => $_[2],
+        patch => $_[3],
+        patchMinor => $_[4]
     };
     return bless($self, __PACKAGE__ );
 }
@@ -216,18 +256,18 @@ sub toVersionString {
     my $self  = shift;
     my $output = '';
     if (defined $self->{major}){
-	$output .= $self->{major};
-	if (defined $self->{minor}){
-	    $output .= '.' . $self->{minor};
-	    if (defined $self->{patch}) {
-		if (HTTP::UA::Parser::Utils::startsWithDigit($self->{patch})) { $output .= '.'; }
-		$output .= $self->{patch};
-		if (defined $self->{patchMinor}) {
-		    if (HTTP::UA::Parser::Utils::startsWithDigit($self->{patchMinor})) { $output .= '.'; }
-		    $output .= $self->{patchMinor};
-		}
-	    }
-	}
+        $output .= $self->{major};
+        if (defined $self->{minor}){
+            $output .= '.' . $self->{minor};
+            if (defined $self->{patch}) {
+                if (HTTP::UA::Parser::Utils::startsWithDigit($self->{patch})) { $output .= '.'; }
+                $output .= $self->{patch};
+                if (defined $self->{patchMinor}) {
+                    if (HTTP::UA::Parser::Utils::startsWithDigit($self->{patchMinor})) { $output .= '.'; }
+                    $output .= $self->{patchMinor};
+                }
+            }
+        }
     }
     return $output;
 }
@@ -236,16 +276,18 @@ sub toString {
     my $self = shift;
     my $suffix = $self->toVersionString();
     if ($suffix){
-	$suffix = ' ' . $suffix;
+        $suffix = ' ' . $suffix;
     }
     return $self->family . $suffix;
 }
 
-sub family	{	shift->{family}		}
-sub major	{	shift->{major}		}
-sub minor  	{	shift->{minor}		}
-sub patch  	{	shift->{patch}		}
-sub patchMinor	{	shift->{patchMinor}	}
+sub family     {   shift->{family}     }
+sub major      {   shift->{major}      }
+sub minor      {   shift->{minor}      }
+sub patch      {   shift->{patch}      }
+sub patchMinor {   shift->{patchMinor} }
+sub brand      {   shift->{brand}      }
+sub model      {   shift->{model}      }
 
 ##=============================================================================
 ## Utils Package
@@ -254,43 +296,20 @@ package HTTP::UA::Parser::Utils;
 
 sub makeParser {
     my $regexes = shift;
-    my $makeParser = shift || \&_makeparser;
+    my $makeParser = shift;
     my @parsers = map {
         $makeParser->($_);
     } @{$regexes};
     
     my $parser = sub {
-	my $ua = shift;
-	my @obj;
-	foreach my $parser (@parsers){
-	    @obj = $parser->($ua);
-	    return HTTP::UA::Parser::Base->new(@obj) if $obj[0];
-	}
-	
-	HTTP::UA::Parser::Base->new();
-    };
-    
-    return $parser;
-}
-
-sub _makeParsers {
-    
-    my ($obj) = shift;
-    my $regexp = $obj->{regex};
-    my $famRep = $obj->{family_replacement};
-    my $majorRep = $obj->{v1_replacement};
-    my $minorRep = $obj->{v2_replacement};
-    my $patchRep = $obj->{v3_replacement};
-    
-    my $parser = sub {
-        my $str = shift;
-        my @m = HTTP::UA::Parser::Utils::exe( $regexp , $str );
-        if (!@m) { return undef; }
-        my $family = defined $famRep ? replace($famRep,qr/\$1/,$m[0]) : $m[0];
-        my $major = defined $majorRep ?  $majorRep : $m[1];
-        my $minor = defined $minorRep ?  $minorRep : $m[2];
-        my $patch = defined $patchRep ?  $patchRep : $m[3];
-        return ($family, $major, $minor, $patch);
+        my $ua = shift;
+        my @obj;
+        foreach my $parser (@parsers){
+            @obj = $parser->($ua);
+            return HTTP::UA::Parser::Base->new(@obj) if $obj[0];
+        }
+        
+        HTTP::UA::Parser::Base->new();
     };
     
     return $parser;
@@ -302,10 +321,30 @@ sub replace {
     return $stringToReplace;
 }
 
-sub exe {
-    my ($expr,$string) = @_;
-    my @m = $string =~ $expr;
-    return @m;
+sub multiReplace {
+    my ($stringToReplace, $matches) = @_;
+    for ($stringToReplace) {
+        s{
+            \$(\d)
+        }{
+            defined @{$matches}[$1-1] ? @{$matches}[$1-1] : '';
+        }egx;
+        s{^\s+|\s+$}{};
+    }
+    if ($stringToReplace eq '') {
+        undef $stringToReplace;
+    }
+    return $stringToReplace;
+}
+
+# precompile regex
+sub regex {
+    my ($expr, $flag) = @_;
+    $flag = $flag || '';
+    if ($flag eq "i") {
+        return qr{$expr}i;
+    }
+    return qr{$expr};
 }
 
 sub startsWithDigit {
@@ -325,16 +364,16 @@ sub getPath {
 1;
 
 __END__
-
-=pod
-
 =head1 NAME
 
 HTTP::UA::Parser - Perl User Agent Parser
 
+=for html
+<a href="https://travis-ci.org/ua-parser/uap-perl"><img src="https://travis-ci.org/ua-parser/uap-perl.svg?branch=master"></a>
+
 =head1 DESCRIPTION
 
-Perl port of the ua-parser project - L<https://github.com/tobie/ua-parser>.
+Perl port of the ua-parser project - L<https://github.com/ua-parser>
 
 =head1 SYNOPSIS
 
@@ -356,6 +395,8 @@ Perl port of the ua-parser project - L<https://github.com/tobie/ua-parser>.
     print $r->os->patch;              # -> undef
     
     print $r->device->family;         # -> "iPhone"
+    print $r->device->brand;          # -> "Apple"
+    print $r->device->model;          # -> "iPhone"
     
 =head1 Methods
 
@@ -379,7 +420,7 @@ Parsers operating system part of the user agent
 
 Parses device part of the user agent
 
-=head1 Strigify Methods
+=head1 Stringify Methods
 
 Methods to print results as strings
 
@@ -413,6 +454,14 @@ returns versions patch part of os/browser
 
 returns version patch minor part of os/browser
 
+=item brand()
+
+returns brand name of device
+
+=item model()
+
+returns model name of device
+
 =back
     
 =head1 INSTALLATION
@@ -430,16 +479,28 @@ Then install:
 
     % make install
 
+=head1 COMMAND LINE
+
+To update regexes.yaml file from command line
+    
+    % ua_parser -u
+
+To parse some user agent from command line
+
+    % ua_parser -p "some user agent"
+    
+Help usage
+
+    % ua_parser -h
+
 =head1 AUTHOR
 
 Mamod A. Mehyar, E<lt>mamod.mehyar@gmail.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2013 by Mamod A. Mehyar
+Copyright (C) 2015 by Mamod A. Mehyar & other contributors
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.1 or,
 at your option, any later version of Perl 5 you may have available.
-
-=cut
